/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: AntTaskJBI.java 09:01:14 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.plugin.jbiplugin;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.maven.plugin.MojoExecutionException;
import org.objectweb.petals.jbi.descriptor.ComponentDescription;
import org.objectweb.petals.jbi.descriptor.Identification;
import org.objectweb.petals.jbi.descriptor.JBIDescriptor;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorBuilder;
import org.objectweb.petals.jbi.descriptor.JBIDescriptorException;
import org.objectweb.petals.jbi.descriptor.ServiceAssembly;
import org.objectweb.petals.jbi.descriptor.ServiceUnit;

/**
 * This class is used to give the main configuration parameters of all JBI goals
 *
 * @author cdeneux - Capgemini Sud
 */
public abstract class JBIAbstractJMXMojo extends JBIAbstractMojo {

	/**
	 * Hostname of the Petals server
	 *
	 * @parameter expression="localhost"
	 * @editable true
	 * @required
	 * @description Hostname of the Petals server
	 */
	protected String host;

	/**
	 * JMX Port of the Petals server
	 *
	 * @parameter expression="7700"
	 * @editable true
	 * @required
	 * @description Hostname JMX Port of the Petals server
	 */
	protected Integer port;

	/**
	 * JMX username of the Petals server
	 *
	 * @parameter expression=""
	 * @editable true
	 * @description JMX username of the Petals server
	 */
	protected String username;

	/**
	 * Password of the Petals JMX server username
	 *
	 * @parameter expression=""
	 * @editable true
	 * @description Password of the Petals JMX server username
	 */
	protected String password;

	protected Logger log = Logger.getLogger(JBIAbstractJMXMojo.class.getName());

	protected JBIDescriptor jbiDescriptor = null;

	/**
	 * Read and parse the artefact JBIDescriptor
	 *
	 * @throws MojoExecutionException
	 */
	protected void readJbiDescriptor() throws MojoExecutionException {

		this.jbiDescriptor = readJbiDescriptor(project.getArtifact().getFile());

		if (this.jbiDescriptor == null) {
			throw new MojoExecutionException("JBI Descriptor is null");
		}
	}

	protected JBIDescriptor readJbiDescriptor(File file)
			throws MojoExecutionException {
		try {

			ZipFile zipFile = new ZipFile(file);
			ZipEntry jbiDescriptorZipEntry = zipFile
					.getEntry("META-INF/jbi.xml");
			InputStream jbiDescriptorInputStream = zipFile
					.getInputStream(jbiDescriptorZipEntry);

			// Load the JBI descriptor
            return JBIDescriptorBuilder.getInstance()
                    .validateAndBuildDescriptor(jbiDescriptorInputStream);

		} catch (JBIDescriptorException e) {
			throw new MojoExecutionException("Error loading JBI descriptor", e);
		} catch (IOException e) {
			throw new MojoExecutionException(
					"Error retrieving the JBI descriptor in the JBI archive or loading the jbi xsd schema", e);
		}
	}

	/**
	 * Read component information from the parsed JBI descriptor
	 *
	 * @throws MojoExecutionException
	 */
	protected String getComponentNameFromJBIDescriptor()
			throws MojoExecutionException {
		ComponentDescription componentDescription = this.jbiDescriptor
				.getComponent();
		if (componentDescription == null) {
			throw new MojoExecutionException("JBI Component descriptor is null");
		}

		Identification identification = componentDescription
				.getIdentification();
		if (identification == null) {
			throw new MojoExecutionException(
					"JBI Component descriptor: <identification> is null");
		}
		return identification.getName();
	}

	/**
	 * Read service assembly information from the parsed JBI descriptor.
	 *
	 * @throws MojoExecutionException
	 */
	protected String getServiceAssemblyNameFromJBIDescriptor()
			throws MojoExecutionException {
		ServiceAssembly serviceAssemblyDescription = this.jbiDescriptor
				.getServiceAssembly();
		if (serviceAssemblyDescription == null) {
			throw new MojoExecutionException(
					"JBI Service assembly descriptor is null");
		}

		Identification identification = serviceAssemblyDescription
				.getIdentification();
		if (identification == null) {
			throw new MojoExecutionException(
					"JBI Service assembly descriptor: <identification> is null");
		}
		return identification.getName();
	}

	/**
	 * Retreive all components declared in the service assembly JBI Descriptor
	 *
	 * @throws MojoExecutionException
	 */
	protected List<String> getComponentsFromServiceAssemblyFromJBIDescriptor()
			throws MojoExecutionException {
		ServiceAssembly serviceAssemblyDescription = this.jbiDescriptor
				.getServiceAssembly();
		if (serviceAssemblyDescription == null) {
			throw new MojoExecutionException(
					"JBI Service assembly descriptor is null");
		}

		List<String> usedComponents = new ArrayList<String>();
		List<ServiceUnit> serviceUnits = serviceAssemblyDescription
				.getServiceUnits();
		for (ServiceUnit serviceUnit : serviceUnits) {
			String componentName = serviceUnit.getTargetComponentName();
			if (!usedComponents.contains(componentName)) {
				usedComponents.add(componentName);
			}
		}
		return usedComponents;
	}

}
